import 'package:flutter/material.dart';
import 'package:flutter/scheduler.dart';

class ParalaxListView extends StatefulWidget {
  final double flexibleHeight;
  final Widget background;
  final Widget foreground;
  final List<Widget> chidrens;
  final Widget floatingWidget;
  final EdgeInsetsGeometry padding;
  final ScrollPhysics physics;

  final IndexedWidgetBuilder itemBuilder;
  final int itemCount;
  final bool scrollingBackground;

  ParalaxListView({
    this.flexibleHeight,
    this.background,
    this.foreground,
    this.floatingWidget,
    this.chidrens,
    this.padding,
    this.physics,
    this.itemBuilder,
    this.itemCount,
    this.scrollingBackground = true,
  });

  @override
  _ParalaxListViewState createState() => _ParalaxListViewState();
}

class _ParalaxListViewState extends State<ParalaxListView> {
  ScrollController _scrollController = ScrollController();
  double _backgroundTopPosition = 0;

  @override
  void initState() {
    super.initState();
    _scrollController.addListener(_onScroll);

    SchedulerBinding.instance.addPostFrameCallback((_) {
      _onScroll();
    });
  }

  @override
  void dispose() {
    _scrollController.removeListener(_onScroll);
    super.dispose();
  }

  GlobalKey _floatingWidgetKey = GlobalKey();
  double get _floatingHeight {
    try {
      if (_floatingWidgetKey.currentContext != null) {
        return (_floatingWidgetKey.currentContext.findRenderObject()
                as RenderBox)
            .size
            .height;
      }
    } catch (e) {}

    return 80;
  }

  _onScroll() {
    if (!widget.scrollingBackground) {
      return;
    }

    double floatingHeight = _floatingHeight;
    double release = (floatingHeight * 2) + 32;

    if (_scrollController.offset < floatingHeight + 32) {
      double minus =
          (_scrollController.offset * (_scrollController.offset / release));

      setState(() {
        _backgroundTopPosition = 0 - minus;
      });
    } else {
      if (_backgroundTopPosition > (0 - widget.flexibleHeight)) {
        setState(() {
          _backgroundTopPosition =
              0 - _scrollController.offset + (floatingHeight / 2) + 8;
        });
      } else {
        _backgroundTopPosition =
            0 - _scrollController.offset + (floatingHeight / 2) + 8;
      }
    }
  }

  @override
  Widget build(BuildContext context) {
    double height = widget.flexibleHeight ?? 250;
    List<Widget> childrens = [
      Container(
        width: MediaQuery.of(context).size.width,
        height: height - (_floatingHeight / 2),
        child: SafeArea(
          child: Container(
            child: widget.foreground,
          ),
        ),
      ),
      Container(
        key: _floatingWidgetKey,
        width: MediaQuery.of(context).size.width,
        child: widget.floatingWidget,
      ),
    ];

    if (widget.chidrens != null && widget.chidrens.isNotEmpty) {
      childrens.addAll(widget.chidrens);
    }

    return Stack(
      children: <Widget>[
        Positioned(
          top: _backgroundTopPosition,
          left: 0,
          right: 0,
          child: Container(
            height: height,
            child: widget.background,
          ),
        ),
        widget.itemBuilder != null
            ? ListView.builder(
                padding: widget.padding ?? const EdgeInsets.all(0),
                controller: _scrollController,
                physics: widget.physics,
                itemCount: widget.itemCount + 2,
                itemBuilder: (context, position) {
                  if (position < 2) {
                    return childrens.elementAt(position);
                  }

                  return widget.itemBuilder(context, position - 2);
                },
              )
            : ListView(
                padding: widget.padding ?? const EdgeInsets.all(0),
                controller: _scrollController,
                physics: widget.physics,
                children: childrens,
              ),
      ],
    );
  }
}
